package edu.mtech.esof427.keyboard;

import java.util.*;
import java.io.*;

public class TraceSequence implements Iterable<Trace>
{
    private List<Trace> sequence = new ArrayList<Trace>();

    // Load from a file which has a trace on each line.
    // format = 0 for touches with (x, y) and time
    // format = 1 for touches with (x, y), time, and finger ID
    public TraceSequence(String filename, int format) throws IOException
    {
        BufferedReader reader = new BufferedReader(new FileReader(filename));
        String line = reader.readLine();        
        while (line != null)
        {
            line = line.trim();

            // Lines starting with # are comments we ignore
            if (line.indexOf('#') != 0)
            {
                // Each line contains a sequence of touch events since a single tap may involve more than one event.
                // They are separated on each line by ;
                // The data of the touch itself is separated by ,
                String [] touches = line.split(";");
                Trace trace = new TraceOrientation(ScreenOrientation.Portrait);
                
                for (String touch : touches)
                {
                    if (format == 0)
                    {
                        TouchTime touchTime = new TouchTime(touch);
                        trace.add(touchTime); 
                    }
                    else if (format == 1)
                    {
                        TouchFinger touchFinger = new TouchFinger(touch);
                        trace.add(touchFinger);    
                    }
                }
                sequence.add(trace);
            }            
            line = reader.readLine();
        }
        reader.close(); 
    }

    public int size()
    {
        return sequence.size();
    }
    
    @Override
    public Iterator<Trace> iterator()
    {
        return sequence.iterator();
    }

}
