package edu.mtech.esof427.keyboard.test;

import static org.junit.Assert.*;

import org.junit.Test;

import edu.mtech.esof427.keyboard.*;

public class RectangleTest
{
    private final static double EPSILON = 1e-8;

    @Test
    public void testGetBoundingBox()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 1.0, 1.0);

        Rectangle box = rect.getBoundingBox();
        assertEquals(rect.getLeft(), box.getLeft(), EPSILON);
        assertEquals(rect.getRight(), box.getRight(), EPSILON);
        assertEquals(rect.getTop(), box.getTop(), EPSILON);
        assertEquals(rect.getBottom(), box.getBottom(), EPSILON);
    }

    @Test
    public void testIsInsideGrid()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 2.0);

        // A grid of points inside the rectangle
        for (double x = 4.1; x < 6.0; x += 0.1)
        {
            for (double y = 4.1; y < 6.0; y += 0.1)
            {
                assertTrue(rect.isInside(new Point(x, y)));
            }
        }
    }

    @Test
    public void testIsOutsideGrid()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 2.0);

        // A grid of points inside the rectangle
        for (double x = 3.1; x < 7.0; x += 0.1)
        {
            for (double y = 3.1; y < 7.0; y += 0.1)
            {
                if (((x < 4.0) || (x > 6.0)) && ((y < 4.0) || (y > 6.0)))
                    assertFalse(rect.isInside(new Point(x, y)));
            }
        }
    }


    @Test
    public void testGetWidth()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 3.0);
        assertEquals(2.0, rect.getWidth(), EPSILON);
    }

    @Test
    public void testGetHeight()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 3.0);
        assertEquals(3.0, rect.getHeight(), EPSILON);
    }

    @Test
    public void testGetLeft()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 3.0);
        assertEquals(4.0, rect.getLeft(), EPSILON);
    }

    @Test
    public void testGetRight()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 3.0);
        assertEquals(6.0, rect.getRight(), EPSILON);
    }

    @Test
    public void testGetTop()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 3.0);
        assertEquals(3.5, rect.getTop(), EPSILON);
    }

    @Test
    public void testGetBottom()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 3.0);
        assertEquals(6.5, rect.getBottom(), EPSILON);
    }

    // One rectangle completely inside the other
    @Test
    public void testIntersectsInside()
    {
        Rectangle rect1 = new Rectangle(new Point(5.0, 5.0), 2.0, 2.0);
        Rectangle rect2 = new Rectangle(new Point(5.0, 5.0), 1.0, 1.0);
        
        assertTrue(rect1.intersects(rect2));
        assertTrue(rect2.intersects(rect1));
    }

    // Two rectangle that are far apart
    @Test
    public void testIntersectsNonoverlap()
    {
        Rectangle rect1 = new Rectangle(new Point(5.0, 5.0), 2.0, 2.0);
        Rectangle rect2 = new Rectangle(new Point(15.0, 15.0), 1.0, 1.0);
        
        assertFalse(rect1.intersects(rect2));
        assertFalse(rect2.intersects(rect1));
    }
    
    // Rectangles that form a cross
    @Test
    public void testIntersectsCross()
    {
        Rectangle rect1 = new Rectangle(new Point(0.0, 0.0), 10.0, 1.0);
        Rectangle rect2 = new Rectangle(new Point(0.0, 0.0), 1.0, 10.0);
        
        assertTrue(rect1.intersects(rect2));
        assertTrue(rect2.intersects(rect1));
    }
    
    // Rectangles that overlap a corner
    @Test
    public void testIntersectsCorner()
    {
        Rectangle rect1 = new Rectangle(new Point(0.0, 0.0), 2.0, 2.0);
        Rectangle rect2 = new Rectangle(new Point(1.0, 1.0), 0.1, 0.1);
        
        assertTrue(rect1.intersects(rect2));
        assertTrue(rect2.intersects(rect1));
    }

    // One rectangle is inside and overlapping on right side
    @Test
    public void testIntersectsRight()
    {
        Rectangle rect1 = new Rectangle(new Point(0.0, 0.0), 2.0, 2.0);
        Rectangle rect2 = new Rectangle(new Point(1.0, 0.0), 0.1, 0.1);
        
        assertTrue(rect1.intersects(rect2));
        assertTrue(rect2.intersects(rect1));
    }

    // One rectangle is inside and overlapping on left side
    @Test
    public void testIntersectsLeft()
    {
        Rectangle rect1 = new Rectangle(new Point(0.0, 0.0), 2.0, 2.0);
        Rectangle rect2 = new Rectangle(new Point(-1.0, 0.0), 0.1, 0.1);
        
        assertTrue(rect1.intersects(rect2));
        assertTrue(rect2.intersects(rect1));
    }
    
    // One rectangle is inside and overlapping on top side
    @Test
    public void testIntersectsTop()
    {
        Rectangle rect1 = new Rectangle(new Point(0.0, 0.0), 2.0, 2.0);
        Rectangle rect2 = new Rectangle(new Point(0.0, -1.0), 0.1, 0.1);
        
        assertTrue(rect1.intersects(rect2));
        assertTrue(rect2.intersects(rect1));
    }
    
    // One rectangle is inside and overlapping on top side
    @Test
    public void testIntersectsBottom()
    {
        Rectangle rect1 = new Rectangle(new Point(0.0, 0.0), 2.0, 2.0);
        Rectangle rect2 = new Rectangle(new Point(0.0, 1.0), 0.1, 0.1);
        
        assertTrue(rect1.intersects(rect2));
        assertTrue(rect2.intersects(rect1));
    }
    
    // Place random rectangles with center point inside first one
    @Test
    public void testIntersectsRandom()
    {
        Rectangle rect1 = new Rectangle(new Point(0.5, 0.5), 1.0, 1.0);
        for (int i = 0; i < 10000; i++)
        {
            
            Rectangle rect2 = new Rectangle(new Point(Math.random(), Math.random()), Math.random(), Math.random());
        
            assertTrue(rect1.intersects(rect2));
            assertTrue(rect2.intersects(rect1));
        }
    }

    @Test
    public void testToString()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 5.0), 2.0, 3.0);
        String str = rect.toString();
        assertTrue(str.length() > 0);
    }

    @Test
    public void testGetCenter()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 6.0), 2.0, 3.0);
        Point center = rect.getCenter();

        assertEquals(5.0, center.getX(), EPSILON);
        assertEquals(6.0, center.getY(), EPSILON);        
    }

    @Test
    public void testGetDistanceToCenter()
    {
        Rectangle rect = new Rectangle(new Point(5.0, 6.0), 2.0, 3.0);
        double dist = rect.getDistanceToCenter(new Point(5.0, 6.0));        
        assertEquals(0.0, dist, EPSILON);

        dist = rect.getDistanceToCenter(new Point(5.0, 16.0));        
        assertEquals(10.0, dist, EPSILON);

        dist = rect.getDistanceToCenter(new Point(15.0, 6.0));        
        assertEquals(10.0, dist, EPSILON);
    }

    @Test
    public void testDoBoundingBoxesIntersectRandom()
    {
        Rectangle rect1 = new Rectangle(new Point(0.5, 0.5), 1.0, 1.0);
        
        for (int i = 0; i < 10000; i++)
        {            
            Rectangle rect2 = new Rectangle(new Point(Math.random(), Math.random()), Math.random(), Math.random());
            
            assertTrue(rect1.doBoundingBoxesIntersect(rect2));
            assertTrue(rect2.doBoundingBoxesIntersect(rect1));
        }
    }

}
